;==========================================================================================
;
;   UNOFFICIAL SNES POWERPAK FIRMWARE V2.00-BETA2 (CODENAMED "SIMBA")
;   (c) 2012 by ManuLwe (http://www.manuloewe.de/)
;
;	*** GRAPHICS DATA ***
;
;==========================================================================================



; ****************************** Defines *******************************

.DEFINE BG1_FONT_4BPP		".\\gfx\\font_4bpp.pic"		; 8192 bytes
.DEFINE BG2_FONT_2BPP		".\\gfx\\font_2bpp.pic"		; 4096 bytes

.DEFINE CURSOR_ARROW		".\\gfx\\cursor_arrow.pic"	; 2x2 tiles = 128 bytes each
.DEFINE CURSOR_SIMBA		".\\gfx\\cursor_simba.pic"	; (files output by gfx2snes might be 512 bytes
.DEFINE CURSOR_REDDOT		".\\gfx\\cursor_reddot.pic"	; in size, only the first 128 will be read anyway)

.DEFINE LOGO_POWERPAK		".\\gfx\\logo_powerpak.pic"	; 8x8 tiles = 2048 bytes each
.DEFINE LOGO_SIMBA		".\\gfx\\logo_simba.pic"
.DEFINE LOGO_SFCDOTS		".\\gfx\\logo_sfcdots.pic"

.DEFINE MUSIC_NOTES		".\\gfx\\music_notes.pic"	; 4x8 tiles = 1024 bytes

.DEFINE PALETTE_POWERPAK	".\\gfx\\logo_powerpak.pal"	; 16 colors = 32 bytes each
.DEFINE PALETTE_SIMBA		".\\gfx\\logo_simba.pal"
.DEFINE PALETTE_SFCDOTS		".\\gfx\\logo_sfcdots.pal"

.DEFINE PALETTE_CURSOR_ARROW	".\\gfx\\cursor_arrow.clr"



; ************************ Font/BG palette data ************************

; There are 32 BG palettes (4 colors = 8 bytes each). Only the first is
; used at all (for both BG1 and BG2). All 32 palettes are directly de-
; fined here.

BG_Palette:
	.db $00					; main backdrop (black)
	.db $00

	.db $63					; font drop shadow (dark grey)
	.db $0C

	.db $00					; unused color
	.db $00

	.db $FF					; font (white)
	.db $7F



.REPEAT 31					; 31 more BG palettes (DON'T REMOVE!)
	.db $FF
	.db $FF

	.db $FF
	.db $FF

	.db $FF
	.db $FF

	.db $FF
	.db $FF
.ENDR



; ************************ Sprite palette data *************************

; There are eight sprite palettes (16 colors = 32 bytes each).
; THE ORDER MUST NOT BE CHANGED (user logo configuration relies on it!)

	.INCBIN PALETTE_POWERPAK		; new PowerPak logo
	.INCBIN PALETTE_SIMBA			; Simba logo, "paw" cursor
	.INCBIN PALETTE_SFCDOTS			; SFC dots logo, "red dot" cursor
	.INCBIN PALETTE_CURSOR_ARROW		; arrow-style cursor, music notes



.REPEAT 128					; four more 32-byte sprite palettes (DON'T REMOVE!)
	.db $FF
.ENDR



; ************************ Font character data *************************

Font2bpp:
	.INCBIN BG2_FONT_2BPP			; 2bpp font for BG2



Font4bpp:
	.INCBIN BG1_FONT_4BPP			; 4bpp font for BG1



; *********************** Sprite character data ************************

; 8192 bytes total. The seemingly odd splitting of the data makes sense
; once it is viewed in the compiled ROM using a tile editor. Navigate
; to the starting offset (usually $B200) and switch to a 16x16 view
; (4bpp mode). Cursors will appear in the upper left, music notes on
; the right, the PowerPak logo in the upper middle (where it used to be
; in v1.0X already :p), Simba in the lower left and SFC dots in the
; lower right.
; All the sprite buffer and user logo configuration math is based upon
; this "original" layout, so it made sense to me to preserve it while
; at the same time being able to edit graphics files individually.
; (Dashed lines below appear after every 16 tiles = 512 bytes, BTW.)

SpriteTiles:

.INCBIN CURSOR_ARROW	READ 64

.REPEAT 64					; two empty tiles (1 tile = 32 bytes)
	.db $00
.ENDR

.INCBIN LOGO_POWERPAK	READ 256
.INCBIN MUSIC_NOTES	READ 128

; ----------------------------------------------

.INCBIN CURSOR_ARROW	SKIP 64   READ 64

.REPEAT 64					; two empty tiles
	.db $00
.ENDR

.INCBIN LOGO_POWERPAK	SKIP 256  READ 256
.INCBIN MUSIC_NOTES	SKIP 128  READ 128

; ----------------------------------------------

.INCBIN CURSOR_SIMBA	READ 64
.INCBIN CURSOR_REDDOT	READ 64
.INCBIN LOGO_POWERPAK	SKIP 512  READ 256
.INCBIN MUSIC_NOTES	SKIP 256  READ 128

; ----------------------------------------------

.INCBIN CURSOR_SIMBA	SKIP 64   READ 64
.INCBIN CURSOR_REDDOT	SKIP 64   READ 64
.INCBIN LOGO_POWERPAK	SKIP 768  READ 256
.INCBIN MUSIC_NOTES	SKIP 384  READ 128

; ----------------------------------------------

.REPEAT 128					; four empty tiles
	.db $00
.ENDR

.INCBIN LOGO_POWERPAK	SKIP 1024 READ 256
.INCBIN MUSIC_NOTES	SKIP 512  READ 128

; ----------------------------------------------

.REPEAT 128					; four empty tiles
	.db $00
.ENDR

.INCBIN LOGO_POWERPAK	SKIP 1280 READ 256
.INCBIN MUSIC_NOTES	SKIP 640  READ 128

; ----------------------------------------------

.REPEAT 128					; four empty tiles
	.db $00
.ENDR

.INCBIN LOGO_POWERPAK	SKIP 1536 READ 256
.INCBIN MUSIC_NOTES	SKIP 768  READ 128

; ----------------------------------------------

.REPEAT 128					; four empty tiles
	.db $00
.ENDR

.INCBIN LOGO_POWERPAK	SKIP 1792 READ 256
.INCBIN MUSIC_NOTES	SKIP 896  READ 128

; ----------------------------------------------

.INCBIN LOGO_SIMBA	READ 256
.INCBIN LOGO_SFCDOTS	READ 256

; ----------------------------------------------

.INCBIN LOGO_SIMBA	SKIP 256  READ 256
.INCBIN LOGO_SFCDOTS	SKIP 256  READ 256

; ----------------------------------------------

.INCBIN LOGO_SIMBA	SKIP 512  READ 256
.INCBIN LOGO_SFCDOTS	SKIP 512  READ 256

; ----------------------------------------------

.INCBIN LOGO_SIMBA	SKIP 768  READ 256
.INCBIN LOGO_SFCDOTS	SKIP 768  READ 256

; ----------------------------------------------

.INCBIN LOGO_SIMBA	SKIP 1024 READ 256
.INCBIN LOGO_SFCDOTS	SKIP 1024 READ 256

; ----------------------------------------------

.INCBIN LOGO_SIMBA	SKIP 1280 READ 256
.INCBIN LOGO_SFCDOTS	SKIP 1280 READ 256

; ----------------------------------------------

.INCBIN LOGO_SIMBA	SKIP 1536 READ 256
.INCBIN LOGO_SFCDOTS	SKIP 1536 READ 256

; ----------------------------------------------

.INCBIN LOGO_SIMBA	SKIP 1792 READ 256
.INCBIN LOGO_SFCDOTS	SKIP 1792 READ 256

; **********************************************************************
