;==========================================================================================
;
;   UNOFFICIAL SNES POWERPAK FIRMWARE V2.00-BETA2 (CODENAMED "SIMBA")
;   (c) 2012 by ManuLwe (http://www.manuloewe.de/)
;
;	*** CF MODULE: SB (SRAM BROWSER) ***
;	Code in this file based on v1.0X code written by:
;	- bunnyboy (SNES PowerPak creator), (c) 2009
;
;==========================================================================================



; ****************************** Includes ******************************

	.INCLUDE "mapper.inc"			; MemoryMap, VectorTable, HeaderInfo

	.INCLUDE "variables.asm"		; Global Variables



; ******************************** Main ********************************

Main:
	sep #A_8BIT				; 8 bit Accumulator, 16 bit X/Y
	rep #XY_8BIT

	stz Joy1New				; reset input buttons
	stz Joy1New+1
	stz Joy1Press
	stz Joy1Press+1

	lda #'S'				; SRM extension
	sta exMatch1
	sta exMatch1+1
	sta exMatch1+2
	sta exMatch1+3
	lda #'R'
	sta exMatch2
	sta exMatch2+1
	sta exMatch2+2
	sta exMatch2+3
	lda #'M'
	sta exMatch3
	sta exMatch3+1
	sta exMatch3+2
	sta exMatch3+3

	lda rootDirCluster
	sta sourceCluster
	lda rootDirCluster+1
	sta sourceCluster+1
	lda rootDirCluster+2
	sta sourceCluster+2
	lda rootDirCluster+3
	sta sourceCluster+3

MainLoad:
	jump jCardLoadDir			; root dir

	jump jDirPrintDir			; this includes a PrintClearScreen / SetCursorPos 0, 0



SRAMBrowserLoop:
	wai

UpCheck:
	lda Joy1+1
	and #%00001000
	beq UpDone

UpPressed:
	lda cursorYCounter
	bne UpDone
	lda scrollYCounter
	bne UpDone

UpScroll:
	lda #$08
	sta speedScroll
	lda #$01
	sta speedCounter
	jump jScrollUp

	ldx #$0015				; 21 frames
-
	wai
	lda Joy1+1
	and #%00001000
	beq UpDone
	dex
	cpx #$0000
	bne -

UpHeld:
	lda cursorYCounter
	bne UpDone
	lda scrollYCounter
	bne UpDone

UpScroll2:
	lda #$08
	sta speedScroll
	lda #$01
	sta speedCounter
	jump jScrollUp

	ldx #$0003				; 3 frames
-
	wai
	lda Joy1+1
	and #%00001000
	beq UpDone
	dex
	cpx #$0000
	bne -
	bra UpHeld

UpDone:



DownCheck:
	lda Joy1+1
	and #%00000100
	beq DownDone

DownPressed:
	lda cursorYCounter
	bne DownDone
	lda scrollYCounter
	bne DownDone

DownScroll:
	lda #$08
	sta speedScroll
	lda #$01
	sta speedCounter
	jump jScrollDown

	ldx #$0015				; 21 frames
-
	wai
	lda Joy1+1
	and #%00000100
	beq DownDone
	dex
	cpx #$0000
	bne -

DownHeld:
	lda cursorYCounter
	bne DownDone
	lda scrollYCounter
	bne DownDone

DownScroll2:
	lda #$08
	sta speedScroll
	lda #$01
	sta speedCounter
	jump jScrollDown

	ldx #$0003				; 3 frames
-
	wai
	lda Joy1+1
	and #%00000100
	beq DownDone
	dex
	cpx #$0000
	bne -
	bra DownHeld

DownDone:



LeftCheck:
	lda Joy1Press+1
	and #%00000010
	beq LeftDone

LeftPressed:
	lda cursorYCounter
	bne LeftDone
	lda scrollYCounter
	bne LeftDone

LeftScroll:
	lda #$04
	sta speedScroll
	lda #$02
	sta speedCounter
	jump jScrollUp

LeftDone:



RightCheck:
	lda Joy1Press+1
	and #%00000001
	beq RightDone

RightPressed:
	lda cursorYCounter
	bne RightDone
	lda scrollYCounter
	bne RightDone

RightScroll:
	lda #$04
	sta speedScroll
	lda #$02
	sta speedCounter
	jump jScrollDown

RightDone:



LCheck:
	lda Joy1New
	and #%00100000
	beq LDone

;	jump jPageUp
	jsr PageUp

LDone:



RCheck:
	lda Joy1New
	and #%00010000
	beq RDone

;	jump jPageDown
	jsr PageDown

RDone:



ACheck:						; A = select file / load dir
	lda Joy1New
	and #%10000000
	beq +

	jsr NextModule
+



BCheck:						; B = go up one directory
	lda Joy1New+1
	and #%10000000
	beq ++

	lda sourceCluster			; check if current dir = root dir ...
	cmp rootDirCluster
	bne +
	lda sourceCluster+1
	cmp rootDirCluster+1
	bne +
	lda sourceCluster+2
	cmp rootDirCluster+2
	bne +
	lda sourceCluster+3
	cmp rootDirCluster+3
	beq ++					; ... if yes, don't do anything
+
	lda #$00				; otherwise, load entry $0001, which is
	sta selectedEntry+1			; always "/.." except for when already in root dir
	lda #$01
	sta selectedEntry

	jump jDirGetEntry

	jump jNextDir
++

	brl SRAMBrowserLoop



NextModule:
	jump jDirGetEntry			; get selected entry

	lda tempEntry.tempDirFlag
	cmp #$01
	bne NextModuleLoad
	jump jNextDir
rts



NextModuleLoad:
	ldy #$0000

CopyGameNameLoop:
	lda tempEntry, y
	sta saveName, y
	iny
	cpy #$0038				; copy 56 chars (WAS-beta1: 53)
	bne CopyGameNameLoop

;	stz saveName+$35			; game name will be NULL terminated in options module

	lda tempEntry.tempCluster
	sta saveName.sCluster
	sta sourceCluster
	lda tempEntry.tempCluster+1
	sta saveName.sCluster+1
	sta sourceCluster+1
	lda tempEntry.tempCluster+2
	sta saveName.sCluster+2
	sta sourceCluster+2
	lda tempEntry.tempCluster+3
	sta saveName.sCluster+3			; copy to save cluster
	sta sourceCluster+3			; copy to source cluster



NextModuleSOI:
	lda #%00000001
	sta CONFIGWRITESTATUS			; no more battery flag

	jump jClearFindEntry

	pla					; clean up the stack as there's no rts from "jsr NextModule"
	pla					; if an SRM file was selected

	lda #'S'
	sta findEntry
	lda nextModule
	sta findEntry+1
	cmp #'I'
	beq NextModuleSI
	brl NextModuleSOIDone



NextModuleSI:
	lda #$00
	sta DMAWRITELO
	sta DMAWRITEHI
	lda #$F8
	sta DMAWRITEBANK

	lda #$FF				; hide cursor
	sta cursorX
	lda #$F0
	sta cursorY

	jump jPrintClearScreen

	SetCursorPos 17, 0			; these msgs are shown when saving a newly selected file
	PrintString "Saving SRAM file to CF card ..."

	lda #kSourceSDRAM
	sta sourceType

	jump jCardWriteFile

	SetCursorPos 17, 0
	PrintString "SRAM file saved successfully!  "	; trailing spaces make "ClearLine 17" unnecessary ;-)

	SetCursorPos 18, 0
	PrintString "Press any button to return to the titlescreen."

-
	lda Joy1New+1				; wait for user input
	and #%11110000				; B, Y, Select, Start
	bne +
	lda Joy1New
	and #%11110000				; A, X, L, R
	bne +
	bra -
+

	jump jPrintClearScreen
	jump jLoadRomVersion

NextModuleSOIDone:
	jump jCardLoadModule			; load options or save module



; ********************** Page-by-page navigation ***********************

PageDown:
	lda filesInDir+1
	cmp #$00
	bne +
	lda filesInDir
	cmp #$19
	bcs +					; if filesInDir < 25, do nothing
rts

+
	rep #A_8BIT				; A = 16 bit

	lda filesInDir				; if filesInDir - selectedEntry < 49, show "last" page
	sec
	sbc selectedEntry
	cmp #$0031
	bcs +
	lda filesInDir				; ... by means of: selectedEntry = filesInDir - 24
	sec
	sbc #$0018
	sta selectedEntry

	bra ShowNextPage
+
	sep #A_8BIT				; A = 8 bit

	jsr SyncPage				; make selectedEntry = entry no. of file at top of screen

	rep #A_8BIT				; A = 16 bit

	lda selectedEntry
	clc
	adc #$0018				; add 24 for "next page"
	sta selectedEntry

ShowNextPage:
	pha					; push selectedEntry (16 bit) onto stack

	sep #A_8BIT				; A = 8 bit

	jsr PrintPage				; new parameters set, print next page

	rep #A_8BIT				; A = 16 bit

	pla					; pull selectedEntry (16 bit) from stack
	sta selectedEntry

	sep #A_8BIT				; A = 8 bit
rts



PageUp:
	lda filesInDir+1
	cmp #$00
	bne +
	lda filesInDir
	cmp #$19
	bcs +					; if filesInDir < 25, do nothing
rts

+
	lda selectedEntry+1
	cmp #$00
	bne +
	lda selectedEntry
	cmp #$18
	bcs +					; if selectedEntry <= 24, show "first" page

	rep #A_8BIT				; A = 16 bit

	lda #$0000
	sta selectedEntry

	bra ShowPrevPage
+
	jsr SyncPage				; make selectedEntry = entry no. of file at top of screen

	rep #A_8BIT				; A = 16 bit

	lda selectedEntry
	sec
	sbc #$0018				; subtract 24 for "previous page"
	bpl +					; if result isn't negative, fine
	lda #$0000				; otherwise, show "first" page as well
+
	sta selectedEntry

ShowPrevPage:
	pha					; push selectedEntry (16 bit) onto stack

	sep #A_8BIT				; A = 8 bit

	jsr PrintPage				; new parameters set, print previous page

	rep #A_8BIT				; A = 16 bit

	pla					; pull selectedEntry (16 bit) from stack
	sta selectedEntry

	sep #A_8BIT				; A = 8 bit
rts



SyncPage:
	lda cursorY
	sec
	sbc #cursorYmin				; subtract indention
	lsr a
	lsr a					; divide by 8 to get the difference between selectedEntry
	lsr a					; and entry no. of the file at the top of the screen
	sta temp

	lda #$00
	sta temp+1

	rep #A_8BIT				; A = 16 bit, required for higher signed number magnitude (+/-16K)

	lda selectedEntry
	sec					; subtract difference so that selectedEntry now corresponds
	sbc temp				; to the file at the top of the screen
	bpl +					; if result isn't negative, fine
	lda #$0000				; otherwise, set selectedEntry to $0000
+
	sta selectedEntry

	sep #A_8BIT				; A = 8 bit
rts



PrintPage:
	lda #minPrintY				; Y indention
	sta temp

	jump jPrintClearScreen
	SetCursorPos 0, 0
-
	inc temp				; go to next print line
	jump jDirPrintEntry

	clc
	lda selectedEntry
	adc #$01
	sta selectedEntry
	lda selectedEntry+1			; increment to next entry index
	adc #$00
	sta selectedEntry+1

	lda temp				; check if printY max reached, jump out
	cmp #maxPrintY
	bne -

	lda #cursorYmin
	sta cursorY				; reset cursor and scrolling parameters
	stz cursorYCounter
	stz cursorYUp
	stz cursorYDown

	stz scrollY
	stz scrollYCounter
	stz scrollYUp
	stz scrollYDown

	stz speedCounter
	stz speedScroll

	lda #insertStandardTop
	sta insertTop
	lda #insertStandardBottom
	sta insertBottom
rts
