;==========================================================================================
;
;   UNOFFICIAL SNES POWERPAK FIRMWARE V2.00-BETA2 (CODENAMED "SIMBA")
;   (c) 2012 by ManuLwe (http://www.manuloewe.de/)
;
;	*** CF MODULE: SO (OPTIONS SCREEN) ***
;	Code in this file based on v1.0X code written by:
;	- bunnyboy (SNES PowerPak creator), (c) 2009
;
;==========================================================================================



; ****************************** Includes ******************************

	.INCLUDE "mapper.inc"			; MemoryMap, VectorTable, HeaderInfo

	.INCLUDE "variables.asm"		; Global Variables



; ******************************** Main ********************************

Main:
	sep #A_8BIT				; 8 bit Accumulator, 16 bit X/Y
	rep #XY_8BIT

	lda #$D5
	sta DMAWRITELO
	lda #$FF
	sta DMAWRITEHI
	lda #$40
	sta DMAWRITEBANK			; destroy header $40FFC0
	lda #$00
	sta DMAREADDATA

	lda #$D5
	sta DMAWRITELO
	lda #$FF
	sta DMAWRITEHI
	lda #$00
	sta DMAWRITEBANK			; destroy header $FFC0
	lda #$00
	sta DMAREADDATA

	lda #$D5
	sta DMAWRITELO
	lda #$7F
	sta DMAWRITEHI
	lda #$00
	sta DMAWRITEBANK			; destroy header $7FC0
	lda #$00
	sta DMAREADDATA

;  lda #%00000011  ;battused, bootmode
;  sta CONFIGWRITESTATUS

	wai
	jump jPrintClearScreen

; ------ V2.00 DEBUG STUFF ---------------------
;	SetCursorPos 22, 1
;	PrintHexNum selectedEntry+1
;	PrintString "-"
;	PrintHexNum selectedEntry
;	PrintString "  "
;	PrintHexNum sourceEntryHi
;	PrintString "-"
;	PrintHexNum sourceEntryLo
;
;	SetCursorPos 2, 22
;	rep #A_8BIT
;	tsc					; print stack pointer (initial value: $1FFF)
;	sep #A_8BIT
;	xba
;	sta temp
;	PrintHexNum temp
;	xba
;	sta temp
;	PrintHexNum temp
; ------ END DEBUG -----------------------------

	SetCursorPos 3, 6
	PrintString "Load battery-backed SRAM ..."

	stz saveName+$35			; NULL terminate save name after 52 characters

	lda #<saveName
	sta strPtr
	lda #>saveName
	sta strPtr+1
	ldy #strPtr

	SetCursorPos 4, 1
	PrintString "%s"

	SetCursorPos 7, 6
	PrintString "Add/edit GameGenie codes ..."

	jsr PrintGGCodes

	SetCursorPos 20, 6
	PrintString "Play!"

	stz gameName+$35			; NULL terminate game name after 52 characters

	lda #<gameName
	sta strPtr
	lda #>gameName
	sta strPtr+1
	ldy #strPtr

	SetCursorPos 21, 1
	PrintString "%s"

	lda #$B8				; put cursor on "Play!" line
	sta cursorY
	lda #$38
	sta cursorX

	stz cursorYCounter			; reset scrolling parameters
	stz cursorYUp
	stz cursorYDown
	stz scrollY
	stz scrollYCounter
	stz scrollYUp
	stz scrollYDown

	stz Joy1New				; reset input buttons
	stz Joy1New+1
	stz Joy1Press
	stz Joy1Press+1



; **************************** Option loops ****************************

; cursorY values:
;	$30 = Load SRAM
;	$50 = Load GG TXT file
;	$60 = GG code 1
;	$70 = GG code 2
;	$80 = GG code 3
;	$90 = GG code 4
;	$A0 = GG code 5
;	$B8 = Play!

; cursorX values:
;	$38 = normal
;	$40 = indent



PlayLoop:
	wai

	lda Joy1New				; check for A button = start game
	and #%10000000
	beq +

	brl StartGame
+

; ---------------------------------------------

	lda Joy1New+1				; check for Start button = start game
	and #%00010000
	beq +

	brl StartGame
+

; ---------------------------------------------

	lda Joy1New+1				; check for B button = return to titlescreen
	and #%10000000
	beq +

	brl BackToIntro
+

; ---------------------------------------------

	lda Joy1New+1				; check for d-pad up
	and #%00001000
	beq +
	lda #$A0				; up pressed, switch to GG code loop
	sta cursorY
	brl CheckIndent
+

; ---------------------------------------------

	lda Joy1New+1				; check for d-pad down
	and #%00000100
	beq +
	lda #$30				; down pressed, switch to save RAM loop
	sta cursorY
	bra SaveRAMLoop
+

; ---------------------------------------------

	lda Joy1Press+1				; check for Select button = show decoded GG codes
	and #%00100000
	beq +					; if Select is released, clear decoded GG code lines

	jsr SelectButton
	bra ++
+
	jsr SelectReleased
++

	bra PlayLoop



SaveRAMLoop:
	wai

	lda Joy1New				; check for A button = launch SRAM browser
	and #%10000000
	beq +

	lda #'O'
	sta nextModule				; return to options after picking SRAM file
	jump jClearFindEntry
	lda #'S'
	sta findEntry
	lda #'B'
	sta findEntry+1

	jump jCardLoadModule			; load SRAM browser
+

; ---------------------------------------------

	lda Joy1New+1				; check for Start button = start game
	and #%00010000
	beq +

	brl StartGame
+

; ---------------------------------------------

	lda Joy1New+1				; check for B button = return to titlescreen
	and #%10000000
	beq +

	brl BackToIntro
+

; ---------------------------------------------

	lda Joy1New+1				; check for d-pad up
	and #%00001000
	beq +
	lda #$B8				; up pressed, switch to play loop
	sta cursorY
	brl PlayLoop
+

; ---------------------------------------------

	lda Joy1New+1				; check for d-pad down
	and #%00000100
	beq +
	lda #$50				; down pressed, switch to GG code loop
	sta cursorY
	bra GGCodeLoop
+

; ---------------------------------------------

	lda Joy1Press+1				; check for Select button = show decoded GG codes
	and #%00100000
	beq +					; if Select is released, clear decoded GG code lines

	jsr SelectButton
	bra ++
+
	jsr SelectReleased
++

	bra SaveRAMLoop



GGCodeLoop:
	wai

	lda Joy1New				; check for A button
	and #%10000000
	beq ++

	lda cursorY				; allow for choosing a GameGenie code TXT file
	cmp #$50
	bne +

	jsr ClearOutGG				; clear out all previously entered codes

	jump jClearFindEntry
	lda #'S'
	sta findEntry
	lda #'G'
	sta findEntry+1

	jump jCardLoadModule			; load GameGenie TXT browser module
+
	jump jGameGenieNextChar			; ... or manipulate GG code
++

; ---------------------------------------------

	lda Joy1New+1				; check for Start button = start game
	and #%00010000
	beq +

	brl StartGame
+

; ---------------------------------------------

	lda Joy1New+1				; check for B button
	and #%10000000
	beq ++

	lda cursorY				; check cursor ($50 = above codes)
	cmp #$50
	bne +

	brl BackToIntro				; return to titlescreen
+
	jump jGameGeniePrevChar			; ... or manipulate GG code
++

; ---------------------------------------------

	lda Joy1New				; check for X/R button
	and #%01010000
	beq +

	lda cursorY				; check cursor ($50 = above codes)
	cmp #$50
	beq +

	jump jGameGenieNextChar			; manipulate GG code
+

; ---------------------------------------------

	lda Joy1New+1				; check for Y/L button
	and #%01000000				; Y
	bne +
	lda Joy1New
	and #%00100000				; L
	bne +
	bra ++
+
	lda cursorY				; check cursor ($50 = above codes)
	cmp #$50
	beq ++
+
	jump jGameGeniePrevChar
++

; ---------------------------------------------

	lda Joy1Press				; check for L+R+X buttons
	and #%01110000
	cmp #%01110000
	bne +

	jsr ClearOutGG				; if all three are pressed, clear out all GG codes
	jsr PrintGGCodes			; and refresh code display
+

; ---------------------------------------------

	lda Joy1Press+1				; check for Select button = show decoded GG codes
	and #%00100000
	beq +					; if Select is released, clear decoded GG code lines

	jsr SelectButton
	bra ++
+
	jsr SelectReleased
++

; ---------------------------------------------

NorthCheck:					; check for d-pad up, set cursorY accordingly
	lda Joy1New+1
	and #%00001000
	bne NorthGG5
	brl SouthCheck

NorthGG5:
	lda cursorY
	cmp #$A0
	bne NorthGG4
	lda #$90
	sta cursorY
	bra CheckIndent

NorthGG4:
	lda cursorY
	cmp #$90
	bne NorthGG3
	lda #$80
	sta cursorY
	bra CheckIndent

NorthGG3:
	lda cursorY
	cmp #$80
	bne NorthGG2
	lda #$70
	sta cursorY
	bra CheckIndent

NorthGG2:
	lda cursorY
	cmp #$70
	bne NorthGG1
	lda #$60
	sta cursorY
	bra CheckIndent

NorthGG1:
	lda cursorY
	cmp #$60
	bne NorthGGLabel
	lda #$50
	sta cursorY
	bra CheckIndent

NorthGGLabel:					; cursorY = $50, switch to SRAM loop
	lda #$30
	sta cursorY
	lda #$38
	sta cursorX
	brl SaveRAMLoop

; ---------------------------------------------

CheckIndent:
	jsr AtIndent				; add indention if on GG code line
	lda temp
	cmp #$01
	bne +

	lda #$40				; on GG code line
	sta cursorX
	brl CheckDone

+
	lda #$38				; not on GG code line
	sta cursorX
	brl CheckDone

; ---------------------------------------------

SouthCheck:					; check for d-pad down, set cursorY accordingly
	lda Joy1New+1
	and #%00000100
	beq WestCheck

SouthStart:
	lda cursorY
	cmp #$A0				; if at GG code #5, switch to "Play!" loop
	bne SouthGG4
	lda #$B8
	sta cursorY
	lda #$38
	sta cursorX
	brl PlayLoop

SouthGG4:
	lda cursorY
	cmp #$90
	bne SouthGG3
	lda #$A0
	sta cursorY
	bra CheckIndent

SouthGG3:
	lda cursorY
	cmp #$80
	bne SouthGG2
	lda #$90
	sta cursorY
	bra CheckIndent

SouthGG2:
	lda cursorY
	cmp #$70
	bne SouthGG1
	lda #$80
	sta cursorY
	bra CheckIndent

SouthGG1:
	lda cursorY
	cmp #$60
	bne SouthGGLabel
	lda #$70
	sta cursorY
	bra CheckIndent

SouthGGLabel:
	lda #$60
	sta cursorY
	bra CheckIndent

; ---------------------------------------------

WestCheck:					; check for d-pad left = move cursor left
	lda Joy1New+1
	and #%00000010
	beq +

	jsr AtIndent				; but only if on GG code line ...
	lda temp
	cmp #$01
	bne CheckDone

	lda cursorX
	cmp #$40				; ... and not at leftmost digit ...
	beq CheckDone

	sec					; cursorX = cursorX - $10
	lda cursorX
	sbc #$10
	sta cursorX
	bra CheckDone
+

; ---------------------------------------------

EastCheck:					; check for d-pad right = move cursor right
	lda Joy1New+1
	and #%00000001
	beq +

	jsr AtIndent				; but only if on GG code line ...
	lda temp
	cmp #$01
	bne CheckDone

	lda cursorX
	cmp #$B0				; ... and not at rightmost digit ...
	beq CheckDone

	clc					; cursorX = cursorX + $10
	lda cursorX
	adc #$10
	sta cursorX
+

; ---------------------------------------------

CheckDone:

	brl GGCodeLoop



AtIndent:
	lda #$01
	sta temp

	lda cursorY				; check if we're above GG code lines
	cmp #$50
	beq +
rts

+
	lda #$00
	sta temp
rts



SelectButton:					; print decoded GG codes
	SetCursorPos 10, 8
	ldy #$0000
	jump jGameGenieDecode
	SetCursorPos 12, 8
	ldy #$0008
	jump jGameGenieDecode
	SetCursorPos 14, 8
	ldy #$0010
	jump jGameGenieDecode
	SetCursorPos 16, 8
	ldy #$0018
	jump jGameGenieDecode
	SetCursorPos 18, 8
	ldy #$0020
	jump jGameGenieDecode
rts



SelectReleased:
	ClearLine 10
	ClearLine 12
	ClearLine 14
	ClearLine 16
	ClearLine 18
rts



ClearOutGG:
	phx
	ldx #$0000
	lda #$10				; $10 = underscore on GG code display
-
	sta gameGenie,x
	inx
	cpx #$0028				; 5 x 8 = 40 characters
	bne -
	plx
rts



PrintGGCodes:					; this prints saved or previously entered codes (!)
	SetCursorPos 9, 7
	ldy #$0000
	jump jGameGeniePrint
	SetCursorPos 11, 7
	ldy #$0008
	jump jGameGeniePrint
	SetCursorPos 13, 7
	ldy #$0010
	jump jGameGeniePrint
	SetCursorPos 15, 7
	ldy #$0018
	jump jGameGeniePrint
	SetCursorPos 17, 7
	ldy #$0020
	jump jGameGeniePrint
rts



StartGame:
	jump jClearFindEntry
	lda #'S'
	sta findEntry
	lda #'M'
	sta findEntry+1
	jump jCardLoadModule			; load mapper module



BackToIntro:
	jump jClearFindEntry
	lda #'S'
	sta findEntry
	lda #'I'
	sta findEntry+1
	jump jPrintClearScreen
	jump jLoadRomVersion
	jump jCardLoadModule			; return to titlescreen
