;==========================================================================================
;
;   UNOFFICIAL SNES POWERPAK FIRMWARE V2.00-BETA2 (CODENAMED "SIMBA")
;   (c) 2012 by ManuLwe (http://www.manuloewe.de/)
;
;	*** CF MODULE: SQ (QUESTIONS) ***
;	Code in this file based on v1.0X code written by:
;	- bunnyboy (SNES PowerPak creator), (c) 2009
;
;==========================================================================================



; ****************************** Includes ******************************

	.INCLUDE "mapper.inc"			; MemoryMap, VectorTable, HeaderInfo

	.INCLUDE "variables.asm"		; Global Variables



; ******************************** Main ********************************

Main:
	sep #A_8BIT				; 8 bit Accumulator, 16 bit X/Y
	rep #XY_8BIT

	wai

	stz Joy1New				; reset input buttons
	stz Joy1New+1
	stz Joy1Press
	stz Joy1Press+1

	wai

	jsr LoadLast

	ClearLine 9				; \ temporarily hide firmware version strings
	ClearLine 10				; /
	ClearLine 11
	ClearLine 12
	ClearLine 13
	ClearLine 14
	ClearLine 15
	ClearLine 16
	ClearLine 21				; \ temporarily hide button assignment hints
	ClearLine 22				; /

	SetCursorPos 11, 0
	PrintString "The game you've played features battery-backed SRAM\n"
	PrintString "to save your progress. Please choose an option:"

	SetCursorPos 14, 1

	lda saveName.sCluster
	bne AutoSaveMenu
	lda saveName.sCluster+1
	bne AutoSaveMenu
	lda saveName.sCluster+2
	bne AutoSaveMenu
	lda saveName.sCluster+3
	bne AutoSaveMenu

	PrintString "(No SRAM file loaded previously, auto-saving disabled)"
	lda #$A0
	sta cursorY
	bra AutoSaveMenuDone

AutoSaveMenu:
	lda #$88				; cursor line = (cursorY - $18) / $08 (in this case, $0E=14)
	sta cursorY
	PrintString "Save SRAM to the previously loaded file:"

	lda #<saveName				; reminder: save name is NULL terminated even before
	sta strPtr				; being written to LASTSNES.TXT (might want to CHANGEME)!
	lda #>saveName
	sta strPtr+1
	ldy #strPtr

	SetCursorPos 15, 1
	PrintString "%s"

AutoSaveMenuDone:

	SetCursorPos 17, 1
	PrintString "Select a file ..."

	SetCursorPos 18, 1
	PrintString "Cancel and discard SRAM!"

	lda #$0D
	sta cursorX



; *************************** Questions loop ***************************

QuestionsLoop:
	wai

	lda Joy1New				; check for A button = make a selection
	and #%10000000
	beq +

	jsr ChooseButton
+

; ---------------------------------------------

	lda Joy1New+1				; check for d-pad up
	and #%00001000
	beq +

	jsr PrevButton				; up pressed
+

; ---------------------------------------------

	lda Joy1New+1				; check for d-pad down
	and #%00000100
	beq +

	jsr NextButton				; down pressed
+

	bra QuestionsLoop



; ************************** Button handling ***************************

NextButton:					; $88 -> $A0 -> $A8
	lda cursorY
	clc
	adc #$08
	sta cursorY
	cmp #$B0
	bne NextButtonDone
	lda #$88
	sta cursorY
	bra NextButtonDone2

NextButtonDone:
	lda cursorY
	cmp #$90
	bne NextButtonDone2
	lda #$A0
	sta cursorY

NextButtonDone2:

rts



PrevButton:					; $A8 -> $A0 -> $88
	lda cursorY
	sec
	sbc #$08
	sta cursorY
	cmp #$80
	bne PrevButtonDone
	lda #$A8
	sta cursorY
	bra PrevButtonDone2

PrevButtonDone:
	lda cursorY
	cmp #$98
	bne PrevButtonDone2
	lda #$88
	sta cursorY

PrevButtonDone2:

rts



ChooseButton:
	lda cursorY
	cmp #$88				; if at "Save to previous file", do just that :-)
	beq AutoSave

	cmp #$A0				; if at "Select a file", go to SRAM browser
	beq NextModuleSB

	cmp #$A8				; if at "Cancel" ...
	bne ChooseButtonDone
	lda #$01				; ... turn off the battery info ...
	sta CONFIGWRITESTATUS
	bra NextModuleSI			; ... and go to intro

ChooseButtonDone:

rts



; ********************* Save & load another module *********************

AutoSave:
	lda saveName.sCluster
	bne AutoSaveContinue
	lda saveName.sCluster+1
	bne AutoSaveContinue
	lda saveName.sCluster+2
	bne AutoSaveContinue
	lda saveName.sCluster+3
	bne AutoSaveContinue
rts



AutoSaveContinue:
	jsr SaveFile
-
	lda Joy1New+1				; wait for user input
	and #%11110000				; B, Y, Select, Start
	bne +
	lda Joy1New
	and #%11110000				; A, X, L, R
	bne +
	bra -
+

NextModuleSI:
	lda #$FF				; hide cursor
	sta cursorX
	lda #$F0
	sta cursorY

	jump jClearFindEntry

	pla					; clean up the stack as there's no rts from "jsr ChooseButton"
	pla

	lda #'S'
	sta findEntry
	lda #'I'
	sta findEntry+1

	jump jPrintClearScreen
	jump jLoadRomVersion
	jump jCardLoadModule			; load intro module (SI.MAP)



NextModuleSB:
	lda #$FF				; hide cursor
	sta cursorX
	lda #$F0
	sta cursorY

	jump jPrintClearScreen

	SetCursorPos 11, 11
	PrintString "Loading ..."

	lda #'I'
	sta nextModule				; go to intro when done

	jump jClearFindEntry

	pla					; clean up the stack as there's no rts from "jsr ChooseButton"
	pla

	lda #'S'
	sta findEntry
	lda #'B'
	sta findEntry+1

	jump jCardLoadModule			; go to SRAM browser (SB.MAP)



; *********************** Reload last game info ************************

LoadLast:					; load LASTSNES.TXT file
	lda #'T'
	sta exMatch1
	sta exMatch1+1
	sta exMatch1+2
	sta exMatch1+3

	sta exMatch3
	sta exMatch3+1
	sta exMatch3+2
	sta exMatch3+3

	lda #'X'
	sta exMatch2
	sta exMatch2+1
	sta exMatch2+2
	sta exMatch2+3

	lda baseDirCluster			; "POWERPAK" dir start
	sta sourceCluster
	lda baseDirCluster+1
	sta sourceCluster+1
	lda baseDirCluster+2
	sta sourceCluster+2
	lda baseDirCluster+3
	sta sourceCluster+3

	jump jClearFindEntry			; find LASTSNES.TXT in "POWERPAK" dir
	lda #'L'
	sta findEntry
	lda #'A'
	sta findEntry+1
	lda #'S'
	sta findEntry+2
	lda #'T'
	sta findEntry+3
	lda #'S'
	sta findEntry+4
	lda #'N'
	sta findEntry+5
	lda #'E'
	sta findEntry+6
	lda #'S'
	sta findEntry+7

	jump jCardLoadDir			; "POWERPAK" dir

	jump jDirFindEntry			; get first cluster
	lda tempEntry.tempCluster
	sta sourceCluster
	lda tempEntry.tempCluster+1
	sta sourceCluster+1
	lda tempEntry.tempCluster+2
	sta sourceCluster+2
	lda tempEntry.tempCluster+3
	sta sourceCluster+3

	lda #<sectorBuffer1
	sta destLo
	lda #>sectorBuffer1
	sta destHi				; put into sector RAM
	stz destBank
	stz sectorCounter
	stz bankCounter
	jump jClusterToLBA			; sourceCluster -> first sourceSector
	lda #kDestWRAM
	sta destType
	jump jCardReadSector			; sector -> WRAM

	ldx #$0000
	ldy #$0000

LoadLastGameLoop:				; game name and cluster
	lda sectorBuffer1, y
	sta gameName, x
	iny
	inx
	cpx #$80				; 128 bytes
	bne LoadLastGameLoop

	ldx #$0000

LoadLastSaveLoop:				; save name and cluster
	lda sectorBuffer1, y
	sta saveName, x
	iny
	inx
	cpx #$80				; 128 bytes
	bne LoadLastSaveLoop

rts



; ************************* Actual SRAM saving *************************

SaveFile:
	lda #$FF				; hide cursor
	sta cursorX
	lda #$F0
	sta cursorY

	ClearLine 17
	ClearLine 18

	lda #%00000001
	sta CONFIGWRITESTATUS			; no more battery flag

	lda saveName.sCluster			; copy save cluster to source cluster
	sta sourceCluster
	lda saveName.sCluster+1
	sta sourceCluster+1
	lda saveName.sCluster+2
	sta sourceCluster+2
	lda saveName.sCluster+3
	sta sourceCluster+3

	lda #$00
	sta DMAWRITELO
	sta DMAWRITEHI
	lda #$F8
	sta DMAWRITEBANK

	SetCursorPos 17, 0			; these msgs are shown when saving a previously loaded file
	PrintString "Saving SRAM file to CF card ..."

	lda #kSourceSDRAM
	sta sourceType

	jump jCardWriteFile

	SetCursorPos 17, 0
	PrintString "SRAM file saved successfully!  "	; trailing spaces make "ClearLine 17" unnecessary ;-)

	SetCursorPos 18, 0
	PrintString "Press any button to return to the titlescreen."
rts
